﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinSchedule;


namespace WS_AppointmentSelection
{
    public partial class Form1 : Form
    {
        Stack<Appointment> selApps = new Stack<Appointment>();
        bool cancelAction = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //add these mappings so that the UltraDayView will listen for Right and Left keys when the appointments are selected
            var rightDummyMapping = new UltraDayViewKeyActionMapping(Keys.Right, UltraDayViewAction.EnterEditMode,
                0, UltraDayViewState.AppointmentsSelected, SpecialKeys.AltCtrl, SpecialKeys.Shift);
            var leftDummyMapping = new UltraDayViewKeyActionMapping(Keys.Left, UltraDayViewAction.EnterEditMode,
                0, UltraDayViewState.AppointmentsSelected, SpecialKeys.AltCtrl, SpecialKeys.Shift);
            ultraDayView1.KeyActionMappings.Add(rightDummyMapping);
            ultraDayView1.KeyActionMappings.Add(leftDummyMapping);

            ultraCalendarInfo1.Owners.Add("John");
            ultraCalendarInfo1.Owners.Add("Kate");

            ultraCalendarInfo1.SelectedDateRanges.Add(DateTime.Today, 3);
            ultraCalendarInfo1.Owners.UnassignedOwner.Visible = false;

            DateTime startTime = DateTime.Today.AddDays(1).AddHours(10);
            ultraCalendarInfo1.Appointments.Add(startTime, "First").Owner = ultraCalendarInfo1.Owners["John"];
            ultraCalendarInfo1.Appointments.Add(startTime.AddMinutes(30), "Second").Owner = ultraCalendarInfo1.Owners["John"];
            ultraCalendarInfo1.Appointments.Add(startTime.AddMinutes(60), "Third").Owner = ultraCalendarInfo1.Owners["John"];
            ultraCalendarInfo1.Appointments.Add(startTime.AddDays(1), "First").Owner = ultraCalendarInfo1.Owners["Kate"];
            ultraCalendarInfo1.Appointments.Add(startTime.AddDays(1).AddMinutes(30), "Second").Owner = ultraCalendarInfo1.Owners["Kate"];
            ultraCalendarInfo1.Appointments.Add(startTime.AddDays(1).AddMinutes(60), "Third").Owner = ultraCalendarInfo1.Owners["Kate"];
            this.ultraDayView1.GroupingStyle = DayViewGroupingStyle.DateWithinOwner;
        }

        private void ultraDayView1_KeyDown(object sender, KeyEventArgs e)
        {
            ultraCalendarInfo1.EventManager.SetEnabled(CalendarInfoEventIds.BeforeSelectedAppointmentsChange, false);
            if (e.Shift && ultraCalendarInfo1.SelectedAppointments.Count > 0 && selApps.Count > 0)
            {
                var apps = ultraCalendarInfo1.Appointments.Cast<Appointment>().ToList();
                switch (e.KeyCode)
                {
                    case Keys.Up:
                        ChangePrevAppStatus(apps);
                        break;
                    case Keys.Down:
                        ChangeNextAppStatus(apps);
                        break;
                    case Keys.Left:
                        ChangeLeftAppStatus(apps);
                        break;
                    case Keys.Right:
                        ChangeRightAppStatus(apps);
                        break;
                    default: break;
                }

            }
            ultraCalendarInfo1.EventManager.SetEnabled(CalendarInfoEventIds.BeforeSelectedAppointmentsChange, true);
        }

        private void ChangeRightAppStatus(List<Appointment> apps)
        {
            var orderedApps = apps.OrderBy(a => a.StartDateTime).ToList();
            var app = orderedApps.FirstOrDefault(a => a.StartDateTime.Date > selApps.Peek().StartDateTime.Date &&
                a.StartDateTime.TimeOfDay == selApps.Peek().StartDateTime.TimeOfDay);
            SetSelected(true, app);
            if (selApps.Count > 1)
            {
                app = orderedApps.FirstOrDefault(a => a.StartDateTime.Date > selApps.Peek().StartDateTime.Date &&
                    a.StartDateTime.TimeOfDay == selApps.Peek().StartDateTime.TimeOfDay);
                SetSelected(false, app);
            }
        }

        private void ChangeLeftAppStatus( List<Appointment> apps)
        {
            
            var orderedApps = apps.OrderByDescending(a => a.EndDateTime).ToList();
            var app = orderedApps.FirstOrDefault(a => a.StartDateTime.Date < selApps.Peek().StartDateTime.Date &&
               a.StartDateTime.TimeOfDay == selApps.Peek().StartDateTime.TimeOfDay);
            SetSelected(true, app);
            if (selApps.Count > 1)
            {
                app = orderedApps.FirstOrDefault(a => a.StartDateTime.Date < selApps.Peek().StartDateTime.Date &&
                    a.StartDateTime.TimeOfDay == selApps.Peek().StartDateTime.TimeOfDay);
                SetSelected(false, app);
            }
        }

        private void ChangeNextAppStatus(List<Appointment> apps)
        {
            var orderedApps = apps.OrderBy(a => a.StartDateTime).ToList();
            var app = orderedApps.FirstOrDefault(a => a.StartDateTime >= selApps.Peek().EndDateTime);
            SetSelected(true, app);
            if (selApps.Count > 1)
            {
                app = orderedApps.FirstOrDefault(a => a.StartDateTime >= selApps.Peek().EndDateTime);
                SetSelected(false, app);
            }

        }

        private void ChangePrevAppStatus( List<Appointment> apps)
        {
            var orderedApps = apps.OrderByDescending(a => a.EndDateTime).ToList();
            var app = orderedApps.FirstOrDefault(a => a.EndDateTime <= selApps.Peek().StartDateTime);
            SetSelected(true, app);
            if (selApps.Count > 1)
            {
                app = orderedApps.FirstOrDefault(a => a.EndDateTime <= selApps.Peek().StartDateTime);
                SetSelected(false, app);
            }

        }

        private void SetSelected(bool selected, Appointment app)
        {
            if (app != null && app.Selected != selected)
            {
                if (selected)
                {
                    app.Selected = selected;
                    selApps.Push(app);
                }
                else
                {
                    if (app == selApps.Skip(1).Take(1).FirstOrDefault())
                    {
                        selApps.Pop().Selected = selected;
                    }
                }
                cancelAction = true;
            }
        }

        private void ultraDayView1_BeforePerformAction(object sender, BeforeDayViewPerformActionEventArgs e)
        {
            if (cancelAction)
            {
                e.Cancel = true;
            }
        }

        private void ultraCalendarInfo1_BeforeSelectedAppointmentsChange(object sender, BeforeSelectedAppointmentsEventArgs e)
        {
            if (e.NewSelectedAppointments.Count > 0)
            {
                foreach (var app in e.NewSelectedAppointments)
                {
                    selApps.Push(app);
                }
            }
        }

    }
}
